package com.ruoyi.system.service.impl.taskImpl;

import com.ruoyi.common.annotation.DataSource;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.enums.DataSourceType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.datasource.DynamicDataSourceContextHolder;
import com.ruoyi.system.domain.task.TaskEntity;
import com.ruoyi.system.mapper.taskMapper.TaskMapper;
import com.ruoyi.system.service.taskService.TaskEntityService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@DataSource(value = DataSourceType.GIG_TASK)
public class TaskEntityServiceImpl implements TaskEntityService {
    @Autowired
    private TaskMapper taskMapper;

    /**
     * 根据条件分页查询用户列表
     *
     * @param taskEntity 用户信息
     * @return 用户信息集合信息
     */
    @Transactional
    public List<TaskEntity> selectAdminTaskList(TaskEntity taskEntity) {
        try {
            return taskMapper.selectAdminTaskList(taskEntity);
        } finally {
            DynamicDataSourceContextHolder.clearDataSourceType();
        }
    }
    /**
     * 通过用户ID查询用户
     *
     * @param taskId 用户ID
     * @return 用户对象信息
     */
    @Override
    public TaskEntity selectAdminTaskById(Long taskId)
    {
        return taskMapper.selectAdminTaskById(taskId);
    }

    /**
     * 通过任务ID删除任务
     * @param taskId 任务ID
     * @return
     */
    @Override
    public int deleteAdminTaskById(Long taskId) {
        return taskMapper.deleteAdminTaskById(taskId);
    }

    /**
     * 根据任务id修改任务信息
     * @param taskEntity 任务信息
     * @return
     */
    @Override
    public int updateAdminTaskById(TaskEntity taskEntity) {
        return taskMapper.updateAdminTaskById(taskEntity);
    }

    /**
     * 新增任务
     * @param taskEntity 任务信息
     * @return
     */
    @Override
    public int insertAdminTask(TaskEntity taskEntity) {
        if (!checkTaskNameUnique(taskEntity)) {
            throw new ServiceException("新增任务'" + taskEntity.getTaskName() + "'失败，任务名称已存在");
        }
        // 新增任务信息
        return taskMapper.insertAdminTask(taskEntity);
    }
    /**
     * 校验任务名称是否唯一
     *
     * @param taskEntity 任务信息
     * @return 结果
     */
    @Override
    public boolean checkTaskNameUnique(TaskEntity taskEntity)
    {
        Long userId = StringUtils.isNull(taskEntity.getId()) ? -1L : taskEntity.getId();
        TaskEntity info = taskMapper.checkTaskNameUnique(taskEntity.getTaskName());
        if (StringUtils.isNotNull(info) && info.getId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }
    /**
     * 导入任务数据
     * @param taskLists 任务数据列表
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operName 操作人
     * @return
     */
    @Override
    public String importTaskData(List<TaskEntity> taskLists, Boolean isUpdateSupport, String operName,Long operId) {
        if (StringUtils.isNull(taskLists) || taskLists.size() == 0)
        {
            throw new ServiceException("导入用户数据不能为空！");
        }
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        for (TaskEntity taskEntity : taskLists)
        {
            try
            {
                // 验证是否存在任务名称
                TaskEntity u = taskMapper.selecTaskByTaskName(taskEntity.getTaskName());
                if (StringUtils.isNull(u))
                {
                    taskEntity.setCreateName(operName);
                    taskEntity.setCreateId(operId);
                    taskMapper.insertAdminTask(taskEntity);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、任务名称 " + taskEntity.getTaskName() + " 导入成功");
                }
                else if (isUpdateSupport)
                {
                    taskEntity.setUpdateBy(operName);
                    taskMapper.updateAdminTaskById(taskEntity);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、任务名称 " + taskEntity.getTaskName() + " 更新成功");
                }
                else
                {
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、任务名称 " + taskEntity.getTaskName() + " 已存在");
                }
            }
            catch (Exception e)
            {
                failureNum++;
                String msg = "<br/>" + failureNum + "、任务名称 " + taskEntity.getTaskName() + " 导入失败：";
                failureMsg.append(msg + e.getMessage());
//                log.error(msg, e);
            }
        }
        if (failureNum > 0)
        {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new ServiceException(failureMsg.toString());
        }
        else
        {
            successMsg.insert(0, "数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        }
        return successMsg.toString();
    }

}